@@ -10,12 +10,14 @@ JavaScript::V8::Context - An object in which we can execute JavaScript
   my $context = JavaScript::V8::Context->new();
 
   # Add a function which we can call from JavaScript
-  $context->bind_function(print => sub { print @_; });
+  $context->bind_function(print => sub { print @_ });
 
   my $result = $context->eval($source);
 
 =head1 INTERFACE
 
+=over
+
 =item bind_function ( $name => $subroutine_ref )
 
 Defines a Perl subroutine ($subroutine_ref) as a native function with the given I<$name>.
@@ -26,4 +28,8 @@ Evaluates the JavaScript code given in I<$source> and
 returns the result from the last statement.
 
 If there is a compilation error (such as a syntax error) or an uncaught exception
-is thrown in JavaScript this method returns undef and $@ is set.
+is thrown in JavaScript, this method returns undef and $@ is set.
+
+=back
+
+=cut
